
/* Chrysalide - Outil d'analyse de fichiers binaires
 * context.h - prototypes pour la mise en place d'un contexte de décompilation
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _DECOMP_CONTEXT_H
#define _DECOMP_CONTEXT_H


#include <glib-object.h>


#include "instruction.h"
#include "../arch/archbase.h"
#include "../format/executable.h"
#include "../analysis/blocks/raccess.h"


/**
 * Impossible d'inclure directement :
 *  - ../analysis/routine.h, qui inclut ../arch/instruction.h.
 *  - ../arch/instruction.h, qui inclut ce fichier.
 *
 * On redéclare donc manuellement les types manquants.
 */

/* "../analysis/routine.h" : Représentation générique de routine (instance) */
typedef struct _GBinRoutine GBinRoutine;



#define G_TYPE_DEC_CONTEXT               g_dec_context_get_type()
#define G_DEC_CONTEXT(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), g_dec_context_get_type(), GDecContext))
#define G_IS_DEC_CONTEXT(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_dec_context_get_type()))
#define G_DEC_CONTEXT_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_DEC_CONTEXT, GDecContextClass))
#define G_IS_DEC_CONTEXT_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_DEC_CONTEXT))
#define G_DEC_CONTEXT_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_DEC_CONTEXT, GDecContextClass))



/* Définition d'un contexte de décompilation (instance) */
typedef struct _GDecContext GDecContext;

/* Définition d'un contexte de décompilation (classe) */
typedef struct _GDecContextClass GDecContextClass;


/* Indique le type défini pour un contexte de décompilation. */
GType g_dec_context_get_type(void);

/* Duplique un contexte de compilation. */
GDecContext *g_dec_context_dup(GDecContext *);

/* Définit les informations essentielles à la décompilation. */
void g_dec_context_set_info(GDecContext *, GBinRoutine *, GExeFormat *);

/* Fournit le format binaire associé au contexte. */
GExeFormat *g_dec_context_get_format(const GDecContext *);

/* Définit la liste des registrés utilisés dans plusieurs blocs. */
void g_dec_context_set_awaited(GDecContext *, GRAccessList *);

/* Fournit la liste des registrés utilisés dans plusieurs blocs. */
const GRAccessList *g_dec_context_get_awaited(const GDecContext *);

/* Définit la liste des registrés déjà alloués et attendus. */
void g_dec_context_set_shared_allocs(GDecContext *, GHashTable *);

/* Fournit la liste des registrés déjà alloués et attendus. */
GHashTable *g_dec_context_get_shared_allocs(const GDecContext *);

/* Propage les registres alloués et attendus par la suite. */
void g_dec_context_spread_allocated_shared_regs(GDecContext *, GDecContext *);

/* Fournit le premier élément de la liste des instructions. */
GDecInstruction *g_dec_context_get_decomp_instrs(const GDecContext *tx);

/* Met à jour le premier élément de la liste des instructions. */
void g_dec_context_set_decomp_instrs(GDecContext *, GDecInstruction *);

/* Convertit un registre machine en un pseudo-registre. */
GDecInstruction *g_dec_context_convert_register(GDecContext *, gpointer, bool, vmpa_t);



#endif  /* _DECOMP_CONTEXT_H */
